//+------------------------------------------------------------------+
//|                                                 correlations.mq4 |
//|                                                           mladen |
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      "mladenfx@gmail.com"

#property indicator_separate_window
#property indicator_buffers    8
#property indicator_level1     0
#property indicator_levelcolor DimGray
#property strict

//
//
//
//
//

#import "correlations.dll"
   double Pearson(double& arrayA[], int isSeriesA, double& arrayB[], int isSeriesB, int bars, int shift, int length,int returnValue);
   double Kendall(double& arrayA[], int isSeriesA, double& arrayB[], int isSeriesB, int bars, int shift, int length,int returnValue);
   double Spearman(double& arrayA[], int isSeriesA, double& arrayB[], int isSeriesB, int bars, int shift, int length,int returnValue);
#import

//
//
//
//
//

enum enPrices
{
   pr_close,      // Close
   pr_open,       // Open
   pr_high,       // High
   pr_low,        // Low
   pr_median,     // Median
   pr_typical,    // Typical
   pr_weighted    // Weighted
};
enum enType
{
   ct_pea, // Pearson
   ct_ken, // Kendall's tau
   ct_spe  // Spearman
};
enum enCstype
{
   cs_def, // Indicator itself
   cs_fis, // Fisher
   cs_stu, // Students t
   cs_dif, // Differences (deviations for Kendal's tau - Pearson excluded) 
   cs_exp, // Expectation of differences (only for Spearman)
   cs_dev  // Deviations of differences (only for Spearman)
};

extern string   _           = "parameters";                                               // Dummy parameter (not used)
extern string   Symbols     = "EURJPY;EURUSD;EURCHF;GBPJPY;GBPUSD;USDJPY;USDCAD;USDCHF";  // Symbols to use
extern int      Length      = 50;                                                         // Correlations period
extern enPrices Price       = PRICE_CLOSE;                                                // Price to use
extern enType   Type        = ct_pea;                                                     // Correlation type
extern enCstype SubType     = 0;                                                          // Correlation sub type
extern color    colorFrom   = LimeGreen;                                                  // Starting color for gradient
extern color    colorMiddle = Gold;                                                       // Middle color for gradient
extern color    colorTo     = MediumVioletRed;                                            // Ending color for gradient
extern int      linesWidth  = 0;                                                          // Lines width
extern string   UniqueID    = "_";                                                        // Unique ID

//
//
//
//
//

double buffer1[];
double buffer2[];
double buffer3[];
double buffer4[];
double buffer5[];
double buffer6[];
double buffer7[];
double buffer8[];
string aPairs[];
string sPairs[];

double prices0[];
double prices1[];
double prices2[];
double prices3[];
double prices4[];
double prices5[];
double prices6[];
double prices7[];
double prices8[];

//
//
//
//
//

int    totalSymbols;
string IndicatorFileName;
bool   returnBars;
string prefix      = "NA";
string suffix      = "NA";
string shortName;
string labelName;
string indName;

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

int init()
{
   SetIndexBuffer(0,buffer1);
      returnBars = (_=="returnBars"); if (returnBars) return(0);

      //
      //
      //
      //
      //
      
                                 SetIndexStyle(0,DRAW_LINE,EMPTY,linesWidth,gradientColor(0,4,colorFrom,colorMiddle));
      SetIndexBuffer(1,buffer2); SetIndexStyle(1,DRAW_LINE,EMPTY,linesWidth,gradientColor(1,4,colorFrom,colorMiddle));
      SetIndexBuffer(2,buffer3); SetIndexStyle(2,DRAW_LINE,EMPTY,linesWidth,gradientColor(2,4,colorFrom,colorMiddle));
      SetIndexBuffer(3,buffer4); SetIndexStyle(3,DRAW_LINE,EMPTY,linesWidth,gradientColor(3,4,colorFrom,colorMiddle));
      SetIndexBuffer(4,buffer5); SetIndexStyle(4,DRAW_LINE,EMPTY,linesWidth,gradientColor(1,5,colorMiddle,colorTo));
      SetIndexBuffer(5,buffer6); SetIndexStyle(5,DRAW_LINE,EMPTY,linesWidth,gradientColor(2,5,colorMiddle,colorTo));
      SetIndexBuffer(6,buffer7); SetIndexStyle(6,DRAW_LINE,EMPTY,linesWidth,gradientColor(3,5,colorMiddle,colorTo));
      SetIndexBuffer(7,buffer8); SetIndexStyle(7,DRAW_LINE,EMPTY,linesWidth,gradientColor(4,5,colorMiddle,colorTo));

      //
      //
      //
      //
      //
      
      string subNameF = "";
      string subNameS = "";
      switch (Type)
      {
         case ct_pea:
            indName = "Pearson ";
            SubType=MathMax(MathMin(SubType,2),0);
               switch (SubType)
               {
                  case 0:                           break;
                  case 1: subNameF = "fisher of ";  break;
                  case 2: subNameS = "Students t "; break;
               }
               break;
         case ct_ken:
            indName = "Kendall\'s tau ";
            SubType= MathMax(MathMin(SubType,3),0);
               switch (SubType)
               {
                  case 0:                           break;
                  case 1: subNameF = "fisher of ";  break;
                  case 2: subNameS = "Students t "; break;
                  case 3: subNameS = "deviations "; break;
               }
               break;
         case ct_spe:
            indName = "Spearman\'s rank ";
            SubType= MathMax(MathMin(SubType,5),0);
               switch (SubType)
               {
                  case 1: subNameF = "fisher of ";                  break;
                  case 2: subNameS = "Students t ";                 break;
                  case 3: subNameS = "differences ";                break;
                  case 4: subNameS = "expectation of differences "; break;
                  case 5: subNameS = "deviations of differences ";  break;
               }
               break;
      }
      labelName = indName;
      indName   = subNameF+ indName + subNameS;
      
      //
      //
      //
      //
      //
      
   IndicatorFileName = WindowExpertName();
   shortName         = UniqueID+" "+indName+" ("+(string)Length+","+(string)SubType+")";
   IndicatorShortName(shortName);
   return(0);
}

int deinit()
{
   for (int i=0;i<8;i++) ObjectDelete(shortName+(string)i);
   return(0);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

int start()
{
   int i,r,counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
         int limit = MathMin(Bars-counted_bars,Bars-1);
         if (returnBars) { buffer1[0] = limit; return(0); }
      
   //
   //
   //
   //
   //

      if (suffix=="NA" || prefix=="NA")
      {
         getPrefixSuffix(prefix,suffix);
         Symbols = StringUpperCase(StringTrimLeft(StringTrimRight(Symbols)));
         if (StringSubstr(Symbols,StringLen(Symbols),1) != ";")
                          Symbols = StringConcatenate(Symbols,";");
     
         //
         //
         //
         //
         //

         int  s = 0; i = StringFind(Symbols,";",s);
         string current;
            while (i > 0)
            {
               current = StringSubstr(Symbols,s,i-s);
               if (Symbol()!=(prefix+current+suffix)) 
               {
                  ArrayResize(aPairs,ArraySize(aPairs)+1);
                              aPairs[ArraySize(aPairs)-1] = current;
               }                        
               s = i + 1;
               i = StringFind(Symbols,";",s);
            }
         totalSymbols = MathMin(ArraySize(aPairs),8);
      
         //
         //
         //
         //
         //
      
         ArrayResize(sPairs,totalSymbols);
         for (i=0; i<totalSymbols; i++)
         {
            SetIndexLabel(i,labelName+" "+aPairs[i]);
            sPairs[i] = prefix+aPairs[i]+suffix;
         }         
      }      

      //
      //
      //
      //
      //
   
      if (ArrayRange(prices0,0) != Bars)
      {
         ArrayResize(prices0,Bars); ArrayResize(prices1,Bars);
         ArrayResize(prices2,Bars); ArrayResize(prices3,Bars);
         ArrayResize(prices4,Bars); ArrayResize(prices5,Bars);
         ArrayResize(prices6,Bars); ArrayResize(prices7,Bars);
         ArrayResize(prices8,Bars);
      }      

      //
      //
      //
      //
      //
         
      int window = WindowFind(shortName);
      for (i=0; i<totalSymbols;i++)
      {
         if (sPairs[i]!=_Symbol) limit = (int)MathMax(limit,MathMin(Bars-1,iCustom(sPairs[i],0,IndicatorFileName,"returnBars",0,0)));
         if (ObjectFind(shortName+(string)i) == -1)
         {
               ObjectCreate(shortName+(string)i,OBJ_LABEL,window,0,0);
                  ObjectSet(shortName+(string)i,OBJPROP_XDISTANCE,10);
                  ObjectSet(shortName+(string)i,OBJPROP_YDISTANCE,10+i*14);
                  ObjectSet(shortName+(string)i,OBJPROP_CORNER,1);
                  
                  color theColor = Black;
                  switch(i)
                  {
                     case 0: theColor = gradientColor(0,4,colorFrom,colorMiddle); break;
                     case 1: theColor = gradientColor(1,4,colorFrom,colorMiddle); break;
                     case 2: theColor = gradientColor(2,4,colorFrom,colorMiddle); break;
                     case 3: theColor = gradientColor(3,4,colorFrom,colorMiddle); break;
                     case 4: theColor = gradientColor(1,5,colorMiddle,colorTo);   break;
                     case 5: theColor = gradientColor(2,5,colorMiddle,colorTo);   break;
                     case 6: theColor = gradientColor(3,5,colorMiddle,colorTo);   break;
                     case 7: theColor = gradientColor(4,5,colorMiddle,colorTo);   break;
                  }
                  ObjectSetText(shortName+(string)i,aPairs[i],10,"Courier new bold",theColor);
            }
      }            

   //
   //
   //
   //
   //
            
   for(i=limit, r=Bars-limit-1; i>=0; i--,r++)
   {
      prices0[r] = iMA(NULL,0,1,0,MODE_SMA,(int)Price,i);
      if (totalSymbols>0) { prices1[r] = getPrice(sPairs[0],Price,i); buffer1[i] = getValue(prices1,i); }
      if (totalSymbols>1) { prices2[r] = getPrice(sPairs[1],Price,i); buffer2[i] = getValue(prices2,i); }
      if (totalSymbols>2) { prices3[r] = getPrice(sPairs[2],Price,i); buffer3[i] = getValue(prices3,i); }
      if (totalSymbols>3) { prices4[r] = getPrice(sPairs[3],Price,i); buffer4[i] = getValue(prices4,i); }
      if (totalSymbols>4) { prices5[r] = getPrice(sPairs[4],Price,i); buffer5[i] = getValue(prices5,i); }
      if (totalSymbols>5) { prices6[r] = getPrice(sPairs[5],Price,i); buffer6[i] = getValue(prices6,i); }
      if (totalSymbols>6) { prices7[r] = getPrice(sPairs[6],Price,i); buffer7[i] = getValue(prices7,i); }
      if (totalSymbols>7) { prices8[r] = getPrice(sPairs[7],Price,i); buffer8[i] = getValue(prices8,i); }
   }                           
   return(0);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

double getPrice(string symbol, int type, int i)
{
   if (StringFind(symbol,"USDX")>-1)
         return(usdIndex(type,i));
   else  return(iMA(symbol,0,1,0,MODE_SMA,type,i));
}
double getValue(double& prices[],int i)
{
   switch(Type)
   {
      case 0: return(Pearson(prices0,false,prices,false,Bars,i,Length,(int)SubType));
      case 1: return(Kendall(prices0,false,prices,false,Bars,i,Length,(int)SubType));
      case 2: return(Spearman(prices0,false,prices,false,Bars,i,Length,(int)SubType));
   }
   return(0);
}
//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

string StringUpperCase(string str) { StringToUpper(str); return(str); }

string USDXsymbols[] = {"EURUSD","USDJPY","GBPUSD","USDCAD","USDSEK","USDCHF"};  
double USDXweights[] = {-0.576,-0.136,-0.119,-0.091,-0.042,-0.036};
bool   USDXdivide[]  = {false,true,false,true,true,true};

//
//
//
//
//

double usdIndex(int priceType, int i)
{
   bool   error   = false;   
   double product = 1.0;

   //
   //
   //
   //
   //
   
      for (int k=0; k<6 && !error; k++)
      {
         double price = iMA(prefix+USDXsymbols[k]+suffix,0,1,0,MODE_SMA,priceType,i);
         double value = 0.00;
   
         error = (price==0);
            if (!error)
            {
               if (USDXdivide[k])
                    value = 1/price;
               else value =   price;
               product *= MathPow(value,USDXweights[k]);
            }
      }
   
   //
   //
   //
   //
   //
   
   if (error)
         return(0);
   else  return(product*50.14348112);    
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

#define sectorSize  1936
#define HFILE_ERROR -1
void getPrefixSuffix(string& tprefix, string& tsuffix)
{ 
   int fileHandle = FileOpenHistory("symbols.raw",FILE_BIN|FILE_READ);
   if (fileHandle == HFILE_ERROR) return;
    
   //
   //
   //
   //
   //

   tprefix=""; tsuffix="";
   for(int i=0;; i++)
   {
      FileSeek(fileHandle, sectorSize*i, SEEK_SET); if (FileIsEnding(fileHandle)) { prefix="err"; break; }
      string symbolName = FileReadString(fileHandle,12);
             symbolName = StringSubstr(symbolName, 0);
                   
             //
             //
             //
             //
             //
              
             int pos = StringFind(symbolName,"EURUSD",0);
             if (pos > -1)
             {
                if (pos>0)                         tprefix = StringSubstr(symbolName,0,pos);
                if ((pos+6)<StringLen(symbolName)) tsuffix = StringSubstr(symbolName,(pos+6),0);
                break;
             }     
   } 
   if (fileHandle>-1) FileClose(fileHandle);
}  

//
//
//
//
//

color gradientColor(int step, int totalSteps, color from, color to)
{
   color newBlue  = getColor(step,totalSteps,(from & 0XFF0000)>>16,(to & 0XFF0000)>>16)<<16;
   color newGreen = getColor(step,totalSteps,(from & 0X00FF00)>> 8,(to & 0X00FF00)>> 8) <<8;
   color newRed   = getColor(step,totalSteps,(from & 0X0000FF)    ,(to & 0X0000FF)    )    ;
   return(newBlue+newGreen+newRed);
}
color getColor(int stepNo, int totalSteps, color from, color to)
{
   double step = (from-to)/(totalSteps-1.0);
   return((color)MathRound(from-step*stepNo));
}